package Treex::Unilang;
$Treex::Unilang::VERSION = '2.20151102';
use strict;
use warnings;
use Treex::Core;

1;

__END__
#TODO this is module just to keep synchronized Unilang with Core
=pod

=encoding utf8

=head1 NAME

Treex::Unilang - collection of blocks parametrized by language and language independent

=head1 VERSION

version 2.20151102

=head1 DESCRIPTION

=head1 AUTHOR

Tomáš Kraut <kraut@ufal.mff.cuni.cz>

=head1 COPYRIGHT AND LICENSE

Copyright © 2011 by Institute of Formal and Applied Linguistics, Charles University in Prague

This module is free software; you can redistribute it and/or modify it under the same terms as Perl itself.
